<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Auth;
use Illuminate\Validation\ValidationException;
use App\Repositories\UserRepository;

class AuthController extends Controller
{
    protected $userRepository;

    public function __construct(UserRepository $userRepository)
    {
        $this->userRepository = $userRepository;
    }

    public function login(Request $request)
    {
        try {
            $credentials = $request->validate([
                'email' => 'required|string|email|exists:users,email',
                'password' => 'required|string|min:8',
            ]);

            $user = $this->userRepository->getByEmail($credentials['email']);
            $roles = [];
            $permissions = [];

            if ($user->is_active != TRUE) {
                return response()->json([
                    'message' => 'Disabled user, not authorized to log in.'
                ], 403);
            }

            if (!Hash::check($credentials['password'], $user->password)) {
                throw ValidationException::withMessages([
                    'password' => ['The password is incorrect.']
                ]);
            }

            foreach ($user->roles as $role) {
                array_push($roles, $role->name);
            }

            foreach ($user->getAllPermissions() as $permission) {
                $permissionData = [
                    'name' => $permission->name,
                    'tag' => $permission->tag,
                    'module_id' => $permission->module_id,
                    'module_name' => $permission->module->name,
                ];
                
                array_push($permissions, $permissionData);
            }


            $userData = [
                'id' => $user->id,
                'email' => $user->email,
                'is_active' => $user->is_active,
                'status' =>  $user->status,
                'is_system_admin' => $user->is_system_admin,
                'last_password_update' => $user->last_password_update,
                'profile_photo' => $user->profile_photo,
                'person' => $user->person,
                'roles' => $roles,
                'permissions' => $permissions,
            ];
            
            $token = $user->createToken('auth_token', ['*'], now()->addWeek())->plainTextToken;

            $cookie = cookie('auth_cookie', $token, 60 * 168);

            return response()->json([
                'access_token' => $token,
                'token_type' => 'Bearer',
                'user' => $userData,
            ], 200)->withCookie($cookie);
            
        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
            
        }
    }

    public function logout()
    {
        try {
            auth()->user()->tokens()->delete();

            return response()->json([
                'message' => 'Successfully logged out, all tokens have been removed.',
            ], 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }
}
