<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use App\Repositories\CatalogRepository;
use Throwable;

class CatalogController extends Controller
{
    protected $catalogRepository;

    public function __construct(CatalogRepository $catalogRepository)
    {
        $this->catalogRepository = $catalogRepository;
    }

    public function all()
    {
        try {
            $catalogs = $this->catalogRepository->all();

            return response()->json($catalogs, 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function create(Request $request)
    {
        $validated = $request->validate([
            'name' => 'required|string|max:100|unique:catalogs,name',
            'code' => 'required|string|max:25|unique:catalogs,code',
            'color' => 'nullable|integer',
            'icon' => 'nullable|integer',
        ]);

        try {
            DB::beginTransaction();
            $request['by_user'] = Auth::user()->id;

            $catalog = $this->catalogRepository->save($request);

            DB::commit();

            return response()->json([
                'message' => 'Catálogo creado correctamente.',
                'data' => $catalog,
            ], 201);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function get(Request $request)
    {
        try {
            $catalogId = $request->byId;

            $catalog = $this->catalogRepository->getById($catalogId);

            if (empty($catalog)) {
                return response()->json([
                    'message' => 'Catalog not found.',
                ], 404);
            }

            return response()->json($catalog, 302);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function edit(Request $request)
    {
        $validated = $request->validate([
            'id' => 'required|uuid',
            'name' => 'required|string|max:100',
            'code' => 'required|string|max:25',
            'color' => 'nullable|integer',
            'icon' => 'nullable|integer',
        ]);

        try {    
            DB::beginTransaction();

            $catalog = $this->catalogRepository->getById($request->id);

            if (empty($catalog)) {
                return response()->json([
                    'message' => 'Catalog not found.'
                ], 404);
            }

            $request['by_user'] = Auth::user()->id;

            $catalog = $this->catalogRepository->update($request);

            DB::commit();

            return response()->json([
                'message' => 'Catálogo actualizado correctamente.',
                'data' => $catalog,
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function delete(Request $request)
    {
        $validated = $request->validate([
            'byId' => 'required|uuid'
        ]);

        try {
            $catalog = $this->catalogRepository->getById($request->byId);

            if (empty($catalog)) {
                return response()->json([
                    'message' => 'Catalog not found.'
                ], 404);
            }

            DB::beginTransaction();

            $catalog = $this->catalogRepository->delete(
                $request->byId,
                Auth::user()->id,
            );

            DB::commit();

            return response()->json([
                'message' => 'Successfully catalog deleted.',
                'data' => $catalog,
            ], 200);


        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
            
        }
    }

    public function items(String $id)
    {
        try {
            $catalog = $this->catalogRepository->getById($id);

            if (empty($catalog)) {
                return response()->json(['message' => 'Catalog not found.'], 404);
            }

            if (empty($catalog->items)) {
                return response()->json(['message' => 'No content.'], 204);
            }

            $items = [];

            foreach ($catalog->items as $item) {
                $itemData = [
                    'id' => $item->id,
                    'name' => $item->name,
                    'description' => $item->description,
                    'value' => $item->value,
                ];

                array_push($items, $itemData);
            }

            return response()->json([
                'catalog' => $catalog->name,
                'items' => $items,
            ], 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function createItem(Request $request)
    {
        $validated = $request->validate([
            'catalog_id' => 'required|uuid|exists:catalogs,id',
            'catalog_code' => 'nullable|string|max:25',
            'name' => 'required|string',
            'description' => 'nullable|string',
        ]);

        try {
            DB::beginTransaction();

            $request['by_user'] = Auth::user()->id;

            $item = $this->catalogRepository->saveItem($request);

            DB::commit();

            return response()->json([
                'message' => 'Item creado correctamente.',
                'data' => $item,
            ], 201);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function editItem(Request $request)
    {
        $validated = $request->validate([
            'id' => 'required|uuid|exists:cat_items,id',
            'name' => 'required|string|max:255',
            'description' => 'nullable|string',
        ]);

        try {
            DB::beginTransaction();

            $request['by_user'] = Auth::user()->id;

            $item = $this->catalogRepository->updateItem($request);

            DB::commit();

            return response()->json([
                'message' => 'Item actualizado correctamente.',
                'data' => $item,
            ], 200);
            
        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function deleteItem(Request $request)
    {
        $validated = $request->validate([
            'byId' => 'required|uuid|exists:cat_items,id',
        ]);

        try {
            DB::beginTransaction();

            $item = $this->catalogRepository->getItemById($request->byId);

            if (empty($item)) {
                return response()->json(['message' => 'Item no encontrado.'], 404);
            }

            $deleteItem = $this->catalogRepository->deleteItem(
                $item->id, 
                Auth::user()->id,
            );

            DB::commit();

            return response()->json([
                'message' => 'Item eliminado correctamente.',
                'data' => $deleteItem,
            ], 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function importItems(Request $request)
    {
        $validated = $request->validate([
            'catalog_id' => 'required|uuid',
            'catalog_code' => 'required|string|max:25',
            'items' => 'required|string',
        ]);

        $catalog = $this->catalogRepository->getById($request->catalog_id);

        if (empty($catalog)) {
            return response()->json([
                'message' => 'Catalog not found.',
            ], 404);
        }

        try {
            DB::beginTransaction();

            $items = json_decode($request->items, true);

            foreach ($items as $item) {
                $item['catalog_id'] = $catalog->id;
                $item['catalog_code'] = $request->catalog_code;
                $item['by_user'] = Auth::user()->id;

                $this->catalogRepository->saveItem($item);
            }

            DB::commit();

            return response()->json([
                'message' => 'Items imported successfully.'
            ], 201);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }
}
