<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use App\Repositories\EntityRepository;
use App\Repositories\CatalogRepository;
use Throwable;

class EntityController extends Controller
{
    protected $entityRepository;
    protected $catalogRepository;

    public function __construct(
        EntityRepository $entityRepository,
        CatalogRepository $catalogRepository,
    )
    {
        $this->entityRepository = $entityRepository;
        $this->catalogRepository = $catalogRepository;
    }

    public function all()
    {
        try {
            $entitiesData = $this->entityRepository->all();
            $entities = [];

            foreach ($entitiesData as $entity) {
                $entityData = [
                    'id' => $entity->id,
                    'type_id' => $entity->type->id,
                    'name' => $entity->name,
                    'type_name' => $entity->type->name,
                    'is_active' => $entity->is_active,
                    'status' => $entity->status,
                ];


                array_push($entities, $entityData);
            }

            return response()->json($entities, 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function create(Request $request)
    {
        $validated = $request->validate([
            'type_id' => 'required|uuid',
            'name' => 'required|string|unique:entities,name',
        ]);

        try {
            DB::beginTransaction();

            $entity = $this->entityRepository->save([
                'type_id' => $request->type_id,
                'name' => $request->name,
                'by_user' => Auth::user()->id,
            ]);

            DB::commit();

            return response()->json([
                'message' => 'Successfully created entity.',
                'data' => $entity,
            ], 201);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function get(Request $request)
    {
        $validated = $request->validate([
            'byId' => 'required_without:byType|uuid',
            'byType' => 'required_without:byId|uuid',
        ]);

        try {
            if ($request->byType) {
                $entitiesData = $this->entityRepository->getByType($request->byType);
                $entities = [];

                foreach ($entitiesData as $entityData) {
                    $offices = [];
                    $areas = [];

                    $branchOffices = $this->catalogRepository
                        ->getItemsByCatCodeAndValue('CAT_BRANCH_OFFICES', $entityData->id);

                    foreach ($branchOffices as $branchOffice) {
                        $officeData = ['id' => $branchOffice->id, 'name' => $branchOffice->name];

                        array_push($offices, $officeData);
                    }
                        
                    $workAreas = $this->catalogRepository
                        ->getItemsByCatCodeAndValue('CAT_AREAS', $entityData->id);

                    foreach ($workAreas as $workArea) {
                        $areaData = ['id' => $workArea->id, 'name' => $workArea->name];

                        array_push($areas, $areaData);
                    }

                    $entity = [
                        'id' => $entityData->id,
                        'type_id' => $entityData->entity_type_id,
                        'type_name' => $entityData->entityType->name,
                        'name' => $entityData->name,
                        'branch_offices' => $offices,
                        'work_areas' => $areas,
                        'is_active' => $entityData->is_active,
                        'status' => $entityData->status,
                    ];

                    array_push($entities, $entity);
                }

                return response()->json($entities, 200);

            } else {
                $entity = $this->entityRepository->getById($request->byId);
    
                if (empty($entity)) {
                    return response()->json([
                        'message' => 'Entity not found.'
                    ], 404);
                }
    
                return response()->json($entity, 200);
            }

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function getByType(Request $request)
    {
        $validated = $request->validate([
            'byType' => 'required|uuid',
        ]);

        try {
            $entities = $this->entityRepository->getByType($request->byType);

            return response()->json($entities, 302);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function edit(Request $request)
    {
        $validated = $request->validate([
            'id' => 'required|uuid',
            'type_id' => 'required|uuid',
            'name' => 'required|string|max:255',
        ]);

        try {
            DB::beginTransaction();

            $entity = $this->entityRepository->getById($request->id);

            if (empty($entity)) {
                return response()->json([
                    'message' => 'Entity not found.'
                ], 404);
            }

            $entityData = $this->entityRepository->update([
                'id' => $request->id,
                'type_id' => $request->type_id,
                'name' => $request->name,
                'by_user' => Auth::user()->id,
            ]);

            $entity = [
                'id' => $entityData->id,
                'entity_type_id' => $entityData->entity_type_id,
                'name' => $entityData->name,
                'is_active' => $entityData->is_active,
                'status' => $entityData->status,
            ];

            DB::commit();

            return response()->json([
                'message' => 'Successfully updated entity.',
                'data' => $entity,
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function delete(Request $request)
    {
        $validated = $request->validate([
            'byId' => 'required|uuid'
        ]);

        try {
            DB::beginTransaction();

            $entity = $this->entityRepository->getById($request->byId);

            if (empty($entity)) {
                return response()->json([
                    'message' => 'Entity not found.'
                ], 404);
            }

            $entityData = $this->entityRepository->delete(
                $request->byId,
                Auth::user()->id, 
            );

            $entity = [
                'id' => $entityData->id,
                'entity_type_id' => $entityData->entity_type_id,
                'name' => $entityData->name,
                'is_active' => $entityData->is_active,
                'status' => $entityData->status,
            ];

            DB::commit();

            return response()->json([
                'message' => 'Successfully deleted entity.',
                'data' => $entity,
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }
}
