<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use App\Repositories\ExpenseObjectRepository;
use App\Http\Requests\SaveItemsRequest;
use Throwable;

class ExpenseObjectController extends Controller
{
    protected $expenseObjRepository;

    public function __construct(ExpenseObjectRepository $expenseObjRepository)
    {
        $this->expenseObjRepository = $expenseObjRepository;
    }

    public function all()
    {
        try {
            $expenseObjectsData = $this->expenseObjRepository->all();
            $expenseObjects = [];

            foreach ($expenseObjectsData as $expenseObjectData) {
                $expenseObject = [
                    'id' => $expenseObjectData->id,
                    'type' => $expenseObjectData->type,
                    'category' => $expenseObjectData->category,
                    'group' => $expenseObjectData->group,
                    'provider' => $expenseObjectData->provider,
                    'name' => $expenseObjectData->name,
                    'description' => $expenseObjectData->description,
                    'status' => $expenseObjectData->status,
                ];

                array_push($expenseObjects, $expenseObject);
            }

            return response()->json($expenseObjects, 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function create(Request $request)
    {
        $validated = $request->validate([
            'type_id' => 'required|uuid|exists:cat_items,id',
            'category_id' => 'required|uuid|exists:cat_items,id',
            'group_id' => 'required|uuid|exists:cat_items,id',
            'provider_id' => 'nullable|uuid|exists:cat_items,id',
            'name' => 'required|string|max:255',
        ]);

        try {
            DB::beginTransaction();

            $expenseObject = $this->expenseObjRepository->save([
                'type_id' => $request->type_id,
                'category_id' => $request->category_id,
                'group_id' => $request->group_id,
                'provider_id' => $request->provider_id,
                'name' => $request->name,
                'description' => $request->description,
                'by_user' => Auth::user()->id,
            ]);

            DB::commit();

            return response()->json([
                'message' => 'Successfully Expense Object created.',
                'data' => $expenseObject,
            ], 201);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
            
        }
    }

    public function get(Request $request)
    {
        $validated = $request->validate([
            'byId' => 'required|uuid'
        ]);

        try {
            $expenseObjectData = $this->expenseObjRepository->getById($request->byId);

            if (empty($expenseObjectData)) {
                return response()->json([
                    'message' => 'Expense Object not found.'
                ], 404);
            }

            $expenseObject = [
                'id' => $expenseObjectData->id,
                'type_id' => $expenseObjectData->type_id,
                'type' => $expenseObjectData->type->name,
                'category_id' => $expenseObjectData->category_id,
                'category' => $expenseObjectData->category->name,
                'group_id' => $expenseObjectData->group_id,
                'group' => $expenseObjectData->group->name,
                'provider_id' => $expenseObjectData->provider_id,
                'provider' => $expenseObjectData->provider->name ?? NULL,
                'name' => $expenseObjectData->name,
                'description' => $expenseObjectData->description,
                'status' => $expenseObjectData->status,
            ];

            return response()->json($expenseObject, 302);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function edit(Request $request)
    {
        $validated = $request->validate([
            'id' => 'required|uuid',
            'type_id' => 'required|uuid|exists:cat_items,id',
            'category_id' => 'required|uuid|exists:cat_items,id',
            'group_id' => 'required|uuid|exists:cat_items,id',
            'provider_id' => 'nullable|uuid|exists:cat_items,id',
            'name' => 'required|string|max:255',
        ]);

        $expenseObject = $this->expenseObjRepository->getById($request->id);

        if (empty($expenseObject)) {
            return response()->json([
                'message' => 'Expense object not found.'
            ], 404);
        }

        try {
            DB::beginTransaction();

            $expenseObjectData = $this->expenseObjRepository->update([
                'id' => $request->id,
                'type_id' => $request->type_id,
                'category_id' => $request->category_id,
                'group_id' => $request->group_id,
                'provider_id' => $request->provider_id,
                'name' => $request->name,
                'description' => $request->description,
                'by_user' => Auth::user()->id,
            ]);

            DB::commit();

            $expenseObject = [
                'id' => $expenseObjectData->id,
                'type_id' => $expenseObjectData->type_id,
                'type' => $expenseObjectData->type->name,
                'category_id' => $expenseObjectData->category_id,
                'category' => $expenseObjectData->category->name,
                'group_id' => $expenseObjectData->group_id,
                'group' => $expenseObjectData->group->name,
                'provider_id' => $expenseObjectData->provider_id,
                'provider' => $expenseObjectData->provider->name ?? NULL,
                'name' => $expenseObjectData->name,
                'description' => $expenseObjectData->description,
                'status' => $expenseObjectData->status,
            ];

            return response()->json([
                'message' => 'Successfully expense object updated.',
                'data' => $expenseObject,
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function delete(Request $request)
    {
        $validated = $request->validate(['byId' => 'required|uuid']);

        $expenseObject = $this->expenseObjRepository->getById($request->byId);

        if (empty($expenseObject)) {
            return response()->json([
                'message' => 'Expense object not found.'
            ], 404);
        }

        try {
            DB::beginTransaction();

            $expenseObjectData = $this->expenseObjRepository->delete(
                $request->byId,
                Auth::user()->id,
            );

            DB::commit();

            $expenseObject = [
                'id' => $expenseObjectData->id,
                'type_id' => $expenseObjectData->type_id,
                'type' => $expenseObjectData->type->name,
                'category_id' => $expenseObjectData->category_id,
                'category' => $expenseObjectData->category->name,
                'group_id' => $expenseObjectData->group_id,
                'group' => $expenseObjectData->group->name,
                'provider_id' => $expenseObjectData->provider_id,
                'provider' => $expenseObjectData->provider->name ?? NULL,
                'name' => $expenseObjectData->name,
                'description' => $expenseObjectData->description,
                'status' => $expenseObjectData->status,
            ];

            return response()->json([
                'message' => 'Successfully expense object deleted.',
                'data' => $expenseObject,
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function import(SaveItemsRequest $request)
    {
        try {
            DB::beginTransaction();

            $validated = $request->validated();

            foreach ($validated['expense_objects'] as $expenseObject) {
                $expenseObject['by_user'] = Auth::user()->id;

                $this->expenseObjRepository->save($expenseObject);
            }

            DB::commit();

            return response()->json([
                'message' => 'Successfully imported expense objects.'
            ], 201);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }
}
