<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Repositories\PermissionRepository;
use Throwable;

class PermissionController extends Controller
{
    private $permissionRepository;

    public function __construct(PermissionRepository $permissionRepository)
    {
        $this->permissionRepository = $permissionRepository;
    }

    public function all()
    {
        try {
            $permissions = [];
            $permissionsData = $this->permissionRepository->all();

            foreach ($permissionsData as $permissionData) {
                $permission = [
                    'id' => $permissionData->id,
                    'name' => $permissionData->name,
                    'tag' => $permissionData->tag,
                    'module_id' => $permissionData->module_id,
                    'module_name' => $permissionData->module->name,
                ];

                array_push($permissions, $permission);
            }

            return response()->json($permissions, 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function create(Request $request)
    {
        $validated = $request->validate([
            'name' => 'required|string|max:255|unique:permissions',
            'tag' => 'required|string|max:255|unique:permissions',
            'module_id' => 'required|uuid',
        ]);
        
        try {
            DB::beginTransaction();

            $permission = $this->permissionRepository->save([
                'name' => $request->name,
                'tag' => $request->tag,
                'module_id' => $request->module_id,
            ]);

            DB::commit();

            return response()->json([
                'message' => 'Permiso creado correctamente.',
                'data' => $permission,
            ], 201);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function edit(Request $request)
    {
        $validated = $request->validate([
            'id' => 'required|uuid',
            'name' => 'required|string|max:255',
            'tag' => 'required|string|max:255',
            // 'module_id' => 'required|uuid',
        ]);

        try {
            DB::beginTransaction();
            
            $permission = $this->permissionRepository->update([
                'id' => $request->id,
                'name' => $request->name,
                'tag' => $request->tag,
                'module_id' => $request->module_id,
            ]);

            DB::commit();

            return response()->json([
                'message' => 'Permiso actualizado correctamente.',
                'data' => $permission,
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function byModules()
    {
        try {
            $allModules = $this->permissionRepository->byModules();

            if (empty($allModules)) {
                return response()->json(['message' => 'No content.'], 204);
            }

            $modules = [];

            foreach ($allModules[0]->items as $module) {
                $moduleData = [
                    'module_id' => $module->id,
                    'module_name' => $module->name,
                    'permissions' => $module->permissions,
                ];

                array_push($modules, $moduleData);
            }

            return response()->json($modules, 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function delete(Request $request)
    {
        $validated = $request->validate([
            'byId' => 'required|exists:permissions,id|uuid',
        ]);
        
        $permissionId = $request->byId;

        // try {
            // DB::beginTransaction();

            $permission = $this->permissionRepository->delete($permissionId);

            // DB::commit();

            return response()->json([
                'message' => 'Permiso eliminado correctamente.',
                'data' => $permission,
            ], 200);

        // } catch (Throwable $e) {
            // DB::rollback();

        //     return response()->json(['error' => $e->getMessage()], 500);
        // }
    }
}
