<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use App\Repositories\PersonRepository;
use Throwable;

class PersonController extends Controller
{
    protected $personRepository;

    public function __construct(PersonRepository $personRepository)
    {
        $this->personRepository = $personRepository;
    }

    public function all()
    {
        try {
            $persons = $this->personRepository->all();

            return response()->json($persons, 200);
            
        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
            
        }

        return response()->json($persons, 200);
    }

    public function create(Request $request)
    {
        try {
            $validated = $request->validate([
                'name' => 'required|string|max:255',
                'last_name' => 'required|string|max:255',
            ]);

            DB::beginTransaction();

            $person = $this->personRepository->save([
                'name' => $request->name,
                'last_name' => $request->last_name,
                'by_user' => Auth::user()->id,
            ]);

            DB::commit();

            return response()->json([
                'message' => 'successfully created person.',
                'data' => $person,
            ], 201);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function edit(Request $request)
    {
        $validated = $request->validate([
            'id' => 'required|uuid',
            'name' => 'required|string|max:255',
            'last_name' => 'required|string|max:255',
        ]);

        try {
            $person = $this->personRepository->getById($request->id);

            if (empty($person)) {
                return response()->json([
                    'message' => 'person not found.'
                ], 404);
            }

            DB::beginTransaction();

            $personData = $this->personRepository->update([
                'id' => $request->id,
                'name' => $request->name,
                'last_name' => $request->last_name,
                'by_user' => Auth::user()->id,
            ]);

            $person = [
                'id' => $personData->id,
                'name' => $personData->name,
                'last_name' => $personData->last_name,
                'is_active' => $personData->is_active,
                'status' => $personData->status,
            ];

            DB::commit();

            return response()->json([
                'message' => 'successfully updated person.',
                'data' => $person,
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function delete(Request $request)
    {
        $validated = $request->validate([
            'byId' => 'required|uuid'
        ]);

        try {
            $person = $this->personRepository->getById($request->byId);

            if (empty($person)) {
                return response()->json([
                    'message' => 'person not found.'
                ], 404);
            }

            DB::beginTransaction();

            $personData = $this->personRepository->delete(
                $request->byId,
                Auth::user()->id,
            );

            $person = [
                'id' => $personData->id,
                'name' => $personData->name,
                'last_name' => $personData->last_name,
                'is_active' => $personData->is_active,
                'status' => $personData->status,
            ];

            DB::commit();

            return response()->json([
                'message' => 'successfully deleted person.',
                'data' => $person,
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);

        }
    }
}
