<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Repositories\RoleRepository;
use Throwable;

class RoleController extends Controller
{
    protected $roleRepository;

    public function __construct(RoleRepository $roleRepository)
    {
        $this->roleRepository = $roleRepository;
    }

    public function all()
    {
        try {
            $roles = $this->roleRepository->all();

            return response()->json($roles, 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function create(Request $request)
    {
        $validated = $request->validate([
            'name' => 'required|string|unique:roles,name',
            'color' => 'nullable|integer',
            'icon' => 'nullable|integer',
            'permissions' => 'required|string',
        ]);

        try {
            DB::beginTransaction();

            $role = $this->roleRepository->save($request->except('permissions'));

            $permissions = explode(',', $request->permissions);;
            $role->syncPermissions($permissions);

            DB::commit();

            return response()->json([
                'message' => 'Rol creado correctamente.',
                'data' => $role,
            ], 201);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function get(Request $request)
    {
        $validated = $request->validate([
            'byId' => 'required|uuid'
        ]);

        try {
            $role = $this->roleRepository->getById($request->byId);

            if (empty($role)) {
                return response()->json([
                    'message' => 'role not found.'
                ], 404);
            }

            $permissions = [];

            foreach ($role->permissions as $permission) {
                $permissionData = [
                    'id' => $permission->id,
                    'name' => $permission->name,
                    'tag' => $permission->tag,
                    'module' => $permission->module_id,
                ];

                array_push($permissions, $permissionData);
            }

            $role = [
                'id' => $role->id,
                'name' => $role->name,
                'permissions' => $permissions,
            ];

            return response()->json($role, 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function edit(Request $request)
    {
        $validated = $request->validate([
            'id' => 'required|uuid|exists:roles,id',
            'name' => 'required|string|max:255',
            'color' => 'nullable|integer',
            'icon' => 'nullable|integer',
            'permissions' => 'required|string',
        ]);

        try {
            DB::beginTransaction();

            $role = $this->roleRepository->update($request->except('permissions'));
            $permissions = explode(',', $request->permissions);
            $role->syncPermissions($permissions);

            DB::commit();

            return response()->json([
                'message' => 'Rol actualizado correctamente.',
                'data' => $role,
            ], 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function delete(Request $request)
    {
        $validated = $request->validate([
            'byId' => 'required|uuid'
        ]);

        try {
            DB::beginTransaction();

            $roleData = $this->roleRepository->getById($request->byId);

            if (empty($roleData)) {
                return response()->json(['message' => 'Rol no encontrado'], 404);
            }

            $role = $this->roleRepository->delete($roleData->id);

            DB::commit();

            return response()->json([
                'message' => 'Rol eliminado correctamente.',
                'data' => $role,
            ], 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }
}
