<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Storage;
use App\Repositories\TransactionRepository;
use App\Repositories\TransactionDetailRepository;
use App\Repositories\TransactionItemRepository;
use App\Repositories\NodeRepository;
use App\Repositories\AccountRepository;
use App\Http\Requests\SaveExpenseRequest;
use App\Http\Requests\SaveDispersionRequest;
use App\Http\Requests\SaveTransactionDetailsRequest;
use App\Http\Requests\SaveImportExpenseRequest;
use App\Repositories\VaultBoxRepository;
use App\Repositories\VaultBoxHistoryRepository;
use Carbon\Carbon;
use Throwable;

class TransactionController extends Controller
{
    protected $transactionRepository;
    protected $transactionDetailRepository;
    protected $transactionItemRepository;
    protected $vaultBoxRepository;
    protected $vaultBoxHistoryRepository;
    protected $nodeRepository;
    protected $accountRepository;

    public function __construct(
        TransactionRepository $transactionRepository,
        TransactionDetailRepository $transactionDetailRepository,
        TransactionItemRepository $transactionItemRepository,
        VaultBoxRepository $vaultBoxRepository,
        VaultBoxHistoryRepository $vaultBoxHistoryRepository,
        NodeRepository $nodeRepository,
        AccountRepository $accountRepository,
    ) {
        $this->transactionRepository = $transactionRepository;
        $this->transactionDetailRepository = $transactionDetailRepository;
        $this->transactionItemRepository = $transactionItemRepository;
        $this->vaultBoxRepository = $vaultBoxRepository;
        $this->vaultBoxHistoryRepository = $vaultBoxHistoryRepository;
        $this->nodeRepository = $nodeRepository;
        $this->accountRepository = $accountRepository;
    }

    public function all()
    {
        try {
            $transactions = $this->transactionRepository->all();

            return response()->json($transactions, 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function getByVaultBoxAndDate(Request $request)
    {
        try {
            $transactions = $this->transactionRepository->getByVaultBoxAndDate($request);

            return response()->json($transactions, 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function saveExpense(SaveExpenseRequest $request)
    {
        try {
            DB::beginTransaction();

            $date = $request->date;
            $vaultBox = $this->vaultBoxRepository->getById($request->vault_box_id);
            $request['by_user'] = Auth::user()->id;
            $request['type'] = 'E';
            $expense = $this->save($request);
            $request['transaction_id'] = $expense->id;
            $account = $this->accountRepository->getById($request->account_id);
            $request['bank_id'] = $account->bank_id ?? null;
            
            if (empty($request->nodes) || is_null($request->nodes)) {
                $nodes = null;
            } else {
                $nodes = $this->getNodesInJson(explode(',', $request->nodes));
            }

            $request['nodes'] = $nodes;

            // Historial
            $vaultBoxHistory = $this->vaultBoxHistoryRepository
                ->getById($request->vault_box_history_id);

            if (empty($vaultBoxHistory)) {
                $vaultBoxHistory = $this->vaultBoxHistoryRepository->save(array(
                    'vault_box_id' => $vaultBox->id,
                    'start_date' => $date,
                    'initial_balance' => $vaultBox->current_amount,
                    'final_balance' => 0,
                    'total_income' => 0,
                    'total_expenditure' => 0,
                    'total_spent' => $request->total_amount,
                    'by_user' => $request->by_user,
                ));
            } else {
                $vaultBoxHistory = $this->vaultBoxHistoryRepository
                    ->addExpense(
                        $vaultBoxHistory, array(
                            'amount' => $request->total_amount, 
                            'by_user' => $request->by_user
                        )
                    );
            }

            $request['vault_box_history_id'] = $vaultBoxHistory->id;
            $expenseDetail = $this->saveDetails($request);

            if ($date >= $vaultBoxHistory->start_date &&
                is_null($vaultBoxHistory->end_date) && 
                $vaultBox->current_amount < $request->total_amount
            ) {
                return response()->json([
                    'error' => 'Current balance of the vault or box is insufficient to perform this operation.'
                ], 400);
            }

            // VaultBox
            if ($date >= $vaultBoxHistory->start_date && is_null($vaultBoxHistory->end_date)) {
                $vaultBoxUpdated = $this->vaultBoxRepository
                    ->subtractCurrentAmount(
                        $vaultBox, array(
                            'amount' => $request->total_amount, 
                            'by_user' => $request->by_user
                        )
                    );
            }

            // Items
            $totalCostItems = 0;

            foreach ($request->items as $item) {
                $totalCostItems += $item['single_price'] * $item['quantity'];
            }

            if ($request->total_amount != $totalCostItems) {
                return response()->json([
                    'error' => 'The total amount doesn\'t match the total cost of items.'
                ], 400);
            }

            if (
                ($expenseDetail->is_global && count($request->items) < 2) || 
                ($expenseDetail->is_global == false && count($request->items) > 1)
            ) {
                return response()->json([
                    'error' => 'The number of items doesn\'t match the expense type (global).'
                ], 400);
            }

            $items = $this->saveItems($request->only(['transaction_id', 'items']));

            DB::commit();

            return response()->json([
                'message' => 'Expense created successfully.',
                'expense' => $expense,
                'items' => $items
            ], 201);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function editExpense(SaveExpenseRequest $request)
    {
        try {
            DB::beginTransaction();

            $date = now()->toImmutable()->format('Y-m-d');
            $expense = $this->transactionRepository->getById($request->id);
            $request['by_user'] = Auth::user()->id;
            $account = $this->accountRepository->getById($request->account_id);
            $request['bank_id'] = $account->bank_id;
            $request['transaction_id'] = $expense->id;

            if ($expense->details->type != 'E') {
                return response()->json([
                    'error' => 'This transactions isn\'t an expense.'
                ], 400);
            }

            $oldVaultBox = $this->vaultBoxRepository->getById($expense->details->vault_box_id);
            $newVaultBox = $this->vaultBoxRepository->getById($request->vault_box_id);
            $oldVaultBoxHistory = $this->vaultBoxHistoryRepository->getById($expense->details->vault_box_history_id);
            $newVaultBoxHistory = $this->vaultBoxHistoryRepository->getById($request->vault_box_history_id);
            $difference = $expense->total_amount - $request->total_amount;
            
            if ($request->date >= $newVaultBoxHistory->start_date && $newVaultBox->current_amount + $difference < 0) {
                return response()->json([
                    'error' => 'There isn\'t enough balance in the vault or box to perform this action.',
                ], 400);
            }

            if (is_null($newVaultBoxHistory)) {
                $newVaultBoxHistory = $this->vaultBoxHistoryRepository->save(array(
                    'vault_box_id' => $newVaultBox->id,
                    'start_date' => $date,
                    'initial_balance' => $newVaultBox->current_amount,
                    'final_balance' => 0,
                    'total_income' => 0,
                    'total_expenditure' => 0,
                    'total_spent' => 0,
                    'by_user' => $request->by_user,
                ));
            }

            // Lógica para editar las bóvedas o cajas y su historial
            $request['transaction_detail_id'] = $expense->details->id;
            $request['vault_box_history_id'] = $newVaultBoxHistory->id;
            $expenseUpdate = $this->edit($request);
            $expenseDetailUpdate = $this->editDetails($request);
            
            $this->vaultBoxHistoryRepository->returnEgress($oldVaultBoxHistory, array(
                'amount' => $expense->total_amount,
                'by_user' => $request->by_user
            ));
            $this->vaultBoxHistoryRepository->addExpense($newVaultBoxHistory, array(
                'amount' => $expenseUpdate->total_amount,
                'by_user' => $request->by_user
            ));

            // Items
            $items = $this->editItems($request->only(['transaction_id', 'items']));

            $totalCostItems = 0;

            foreach ($items as $item) {
                $totalCostItems += $item->single_price * $item->quantity;
            }

            if ($request->total_amount != $totalCostItems) {
                return response()->json([
                    'error' => 'The total amount doesn\'t match the total cost of items.'
                ], 400);
            }

            if (
                ($expenseDetailUpdate->is_global && count($items) < 2) || 
                ($expenseDetailUpdate->is_global == false && count($items) > 1)
            ) {
                return response()->json([
                    'error' => 'The number of items doesn\'t match the expense type (global).'
                ], 400);
            }
            
            DB::commit();

            return response()->json([
                'message' => 'Expense updated successfully.',
                'data' => $expenseUpdate,
                'items' => $items
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function importExpenses(SaveImportExpenseRequest $request)
    {
        try {
            DB::beginTransaction();

            $successfully = 0;
            $errors = array();
            
            foreach ($request->expenses as $key => $expense) {
                $vaultBoxHistory = $this->vaultBoxHistoryRepository->getByDate(
                    $expense['vault_box_id'], $expense['date']
                );

                if (empty($vaultBoxHistory)) {
                    $vaultBoxHistory = $this->vaultBoxHistoryRepository->getByOneDate(
                        $expense['vault_box_id'], $expense['date']
                    );
                }
    
                if (empty($vaultBoxHistory)) {
                    $vaultBoxHistory = $this->vaultBoxHistoryRepository->save([
                        'vault_box_id' => $expense['vault_box_id'],
                        'start_date' => $expense['date'],
                        'end_date' => $expense['date'],
                        'initial_balance' => 0,
                        'total_income' => 0,
                        'total_expenditure' => 0,
                        'total_spent' => 0,
                        'by_user' => Auth::user()->id,
                    ]);
                }

                $expense['vault_box_history_id'] = $vaultBoxHistory->id;

                $newExpense = $this->saveExpense(new SaveExpenseRequest($expense));

                if (count($newExpense->original) == 1) {
                    $newExpense->original['expense_key'] = $key;
                    array_push($errors, $newExpense->original);
                }

                $successfully++;
            }

            DB::commit();

            if (empty($errors)) {
                return response()->json([
                    'message' => $successfully . ' of ' . count($request->expenses) . ' expenses imported successfully.'
                ], 200);

            } else {
                return response()->json(['errors' => $errors], 400);

            }

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 200);
        }
    }

    public function saveDispersion(SaveDispersionRequest $request)
    {
        try {
            DB::beginTransaction();

            $vaultBoxParent = $this->vaultBoxRepository->getById($request->vault_box_id);
            $vaultBoxChild = $this->vaultBoxRepository->getById($request->dispersed_to);

            if (!$request->is_refund && $vaultBoxChild->belongs_to != $vaultBoxParent->id) {
                return response()->json([
                    'error' => 'The box doesn\'t belong to the same parent.'
                ], 400);
            }

            if ($vaultBoxParent->current_amount < $request->total_amount) {
                return response()->json([
                    'error' => 'Current balance of the parent is insufficient to perform this operation.'
                ], 400);
            }

            $request['concept'] = $request->concept ?? 'DISPERSIÓN';
            $request['date'] = Date(now());
            $request['entity_id'] = $vaultBoxParent->entity_id;
            $request['by_user'] = Auth::user()->id;
            $request['type'] = $request->is_refund ? 'R' : 'D';

            $dispersion = $this->save($request);
            $request['transaction_id'] = $dispersion->id;

            // Historial
            $date = now()->toImmutable()->format('Y-m-d');
            
            $vaultBoxHistoryParent = $this->vaultBoxHistoryRepository
                ->getOneOpen($vaultBoxParent->id, $date, 'DESC');

            if (empty($vaultBoxHistoryParent)) {
                $vaultBoxHistoryParent = $this->vaultBoxHistoryRepository->save(array(
                    'vault_box_id' => $vaultBoxParent->id,
                    'start_date' => $date,
                    'initial_balance' => $vaultBoxParent->current_amount,
                    'total_income' => 0,
                    'total_expenditure' => $request->total_amount,
                    'total_spent' => 0,
                    'by_user' => $request->by_user,
                ));
            } else {
                $vaultBoxHistoryParent = $this->vaultBoxHistoryRepository
                    ->addEgress(
                        $vaultBoxHistoryParent, array(
                            'amount' => $request->total_amount,        
                            'by_user' => $request->by_user
                        )
                    );
            }

            $vaultBoxHistoryChild = $this->vaultBoxHistoryRepository
                ->getOneOpen($vaultBoxChild->id, $date, 'DESC');

            if (empty($vaultBoxHistoryChild)) {
                $vaultBoxHistoryChild = $this->vaultBoxHistoryRepository->save(array(
                    'vault_box_id' => $vaultBoxChild->id,
                    'start_date' => $date,
                    'initial_balance' => $vaultBoxChild->current_amount,
                    'total_income' => $request->total_amount,
                    'total_expenditure' => 0,
                    'total_spent' => 0,
                    'by_user' => $request->by_user,
                ));
            } else {
                $vaultBoxHistoryChild = $this->vaultBoxHistoryRepository
                    ->addIncome(
                        $vaultBoxHistoryChild, array(
                            'amount' => $request->total_amount, 
                            'by_user' => $request->by_user
                        )
                    );
            }

            $request['vault_box_history_id'] = $vaultBoxHistoryParent->id;
            $request['dispersed_history_id'] = $vaultBoxHistoryChild->id;
            $dispersionDetails = $this->saveDetails($request);

            // VaultBox
            $this->vaultBoxRepository->subtractCurrentAmount(
                $vaultBoxParent, array(
                    'amount' => $request->total_amount, 
                    'by_user' => $request->by_user
                )
            );

            $this->vaultBoxRepository->addCurrentAmount(
                $vaultBoxChild, array(
                    'amount' => $request->total_amount,
                    'by_user' => $request->by_user
                )
            );

            DB::commit();

            return response()->json([
                'message' => 'Dispersion successfully registered.',
                'data' => $dispersion
            ], 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function editDispersion(SaveDispersionRequest $request)
    {
        try {
            DB::beginTransaction();

            $date = now()->toImmutable()->format('Y-m-d');
            $dispersion = $this->transactionRepository->getById($request->id);
            $vaultBox = $this->vaultBoxRepository->getById($dispersion->details->vault_box_id);
            $dispersedTo = $this->vaultBoxRepository->getById($dispersion->details->dispersed_to);
            $difference = $dispersion->total_amount - $request->total_amount;

            if ($dispersion->details->type != 'D') {
                return response()->json([
                    'error' => 'This transactions isn\'t a dispersion.'
                ], 400);
            }

            if ($date != Carbon::parse($dispersion->date)->format('Y-m-d')) {
                return response()->json([
                    'error' => 'It isn\'t possible to edit dispersions from past dates.'
                ], 400);
            }

            if ($vaultBox->current_amount + $difference < 0) {
                return response()->json([
                    'error' => 'Current balance of the parent is insufficient to perform this operation.'
                ], 400);
            }

            $request['by_user'] = Auth::user()->id;
            $dispersionUpdate = $this->edit($request);
            $request['transaction_details_id'] = $dispersionUpdate->details->id;
            $dispersionDetails = $this->saveDetails($request);

            // Historial

            // TODO: Agregar/Restar saldo actual de la bóveda o caja
            $this->vaultBoxRepository->addCurrentAmount($vaultBox, array(
                'amount' => $difference,
                'by_user' => $request->by_user
            ));

            // TODO: Agregar/Restar saldo actual de la caja
            $this->vaultBoxRepository->subtractCurrentAmount($dispersedTo, array(
                'amount' => $difference,
                'by_user' => $request->by_user
            ));

            // TODO: Revertir dispersión del historial
            $vaultBoxHistory = $this->vaultBoxHistoryRepository
                ->getById($dispersion->details->vault_box_history_id);
            
            $dispersedHistory = $this->vaultBoxHistoryRepository
            ->getById($dispersion->details->dispersed_history_id);

            $this->vaultBoxHistoryRepository->returnDispersion(
                $vaultBoxHistory, $dispersedHistory, array(
                    'amount' => $dispersion->total_amount,
                    'by_user' => $request->by_user
                )
            );

            // TODO: Agregar dispersión al historial de la bóveda o caja
            $this->vaultBoxHistoryRepository->addEgress($vaultBoxHistory, array(
                'amount' => $request->total_amount,
                'by_user' => $request->by_user
            ));
            // TODO: Agregar dispersión al historial de la caja
            $this->vaultBoxHistoryRepository->addIncome($dispersedHistory, array(
                'amount' => $request->total_amount,
                'by_user' => $request->by_user
            ));

            DB::commit();

            return response()->json([
                'message' => 'Dispersion successfully registered.',
                'data' => $dispersionUpdate
            ], 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function save(Request $request)
    {   
        try {
            DB::beginTransaction();
            
            $transaction = $this->transactionRepository->save($request);

            DB::commit();

            return $transaction;

        } catch (Throwable $e) {
            DB::rollback();

            return $e->getMessage();

        }
    }

    public function edit(Request $request)
    {
        try {
            DB::beginTransaction();

            $transaction = $this->transactionRepository->update($request);
            
            DB::commit();

            return $transaction;

        } catch (Throwable $e) {
            DB::rollback();

            return $e->getMessage();
            
        }
    }

    public function deleteExpense(Request $request)
    {
        $validated = $request->validate([
            'id' => 'required|uuid',
            'comment' => 'required|string',
        ]);

        try {
            DB::beginTransaction();

            $today = now()->toImmutable()->format('Y-m-d');
            $request['by_user'] = Auth::user()->id;
            $transaction = $this->transactionRepository->getById($request->id);
            
            if ($transaction->details->type != 'E') {
                return response()->json([
                    'error' => 'This transaction isn\'t an expense.',
                ], 400);
            }

            $transaction = $this->transactionRepository->delete($request);
            $vaultBox = $this->vaultBoxRepository->getById($transaction->details->vault_box_id);
            $vaultBoxHistory = $this->vaultBoxHistoryRepository->getById(
                $transaction->details->vault_box_history_id
            );

            if (Carbon::parse($transaction->date)->format('Y-m-d') == $today) {
                $this->vaultBoxRepository->addCurrentAmount($vaultBox, array(
                    'amount' => $transaction->total_amount,
                    'by_user' => $request['by_user']
                ));
            }

            $this->vaultBoxHistoryRepository->returnEgress($vaultBoxHistory, array(
                'amount' => $transaction->total_amount, 
                'by_user' => Auth::user()->id
            ));

            DB::commit();

            return response()->json([
                'message' => 'Transaction successfully deleted.',
                'data' => $transaction,
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function deleteDispersion(Request $request)
    {
        $validated = $request->validate([
            'id' => 'required|uuid',
            'comment' => 'required|string',
        ]);

        try {
            DB::beginTransaction();

            $today = now()->toImmutable()->format('Y-m-d');
            $request['by_user'] = Auth::user()->id;
            $transaction = $this->transactionRepository->getById($request->id);
            
            if ($transaction->details->type != 'D') {
                return response()->json([
                    'error' => 'This transaction isn\'t an dispersion.',
                ], 400);
            }

            $vaultBoxParent = $this->vaultBoxRepository->getById($transaction->details->vault_box_id);
            $vaultBoxHistoryParent = $this->vaultBoxHistoryRepository->getById(
                $transaction->details->vault_box_history_id
            );

            $vaultBoxChild = $this->vaultBoxRepository->getById($transaction->details->dispersed_to);
            $vaultBoxHistoryChild = $this->vaultBoxHistoryRepository->getById(
                $transaction->details->dispersed_history_id
            );

            if (Carbon::parse($transaction->date)->format('Y-m-d') == $today) {
                if ($vaultBoxChild->current_amount < $transaction->total_amount) {
                    return response()->json([
                        'message' => 'The vault or box doesn\'t have sufficient balance to carry out this operation.'
                    ], 400);
                }

                $this->vaultBoxRepository->subtractCurrentAmount($vaultBoxChild, array(
                    'amount' => $transaction->total_amount,
                    'by_user' => $request['by_user']
                ));

                $this->vaultBoxRepository->addCurrentAmount($vaultBoxParent, array(
                    'amount' => $transaction->total_amount,
                    'by_user' => $request['by_user']
                ));
            }

            $transaction = $this->transactionRepository->delete($request);

            $this->vaultBoxHistoryRepository->returnDispersion(
                $vaultBoxHistoryParent,
                $vaultBoxHistoryChild,
                array(
                    'amount' => $transaction->total_amount, 
                    'by_user' => Auth::user()->id
                )
            );

            DB::commit();

            return response()->json([
                'message' => 'Transaction successfully deleted.',
                'data' => $transaction,
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    // Detalles
    public function saveDetails(Request $request)
    {
        try {
            DB::beginTransaction();

            $transactionDetail = $this->transactionDetailRepository->save($request->except('proof_transaction'));
            
            DB::commit();

            return $transactionDetail;

            DB::afterCommit(function () use ($request) {
                if ($request->hasFile('proof_transaction')) {
                    $request = $request->only('proof_transaction');
                    $request['detail_id'] = $transactionDetail->id;
    
                    $this->uploadProofTransaction($request);
                }
            });

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function editDetails(Request $request)
    {
        try {
            DB::beginTransaction();

            $transactionDetail = $this->transactionDetailRepository->getById($request->transaction_detail_id);
            $transactionDetailUpdated = $this->transactionDetailRepository->update(
                $transactionDetail, $request->except('proof_transaction')
            );
            
            DB::commit();

            return $transactionDetailUpdated;

            DB::afterCommit(function () use ($request) {
                if ($request->hasFile('proof_transaction')) {
                    $request = $request->only('proof_transaction');
                    $request['detail_id'] = $transactionDetail->id;
    
                    $this->uploadProofTransaction($request);
                }
            });

        } catch (Throwable $e) {
            DB::rollback();

            return $e->getMessage();

        }
    }

    // Items
    public function getItems(String $id)
    {
        try {
            $transaction = $this->transactionRepository->getById($id);

            if (empty($transaction)) {
                return response()->json([
                    'message' => 'Transaction not found.'
                ], 404);
            }

            $items = $this->transactionItemRepository->getAllOfTransaction($transaction->id);

            return response()->json($items, 302);
            
        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function saveItems(Array $request)
    {
        try {
            DB::beginTransaction();

            $request['by_user'] = Auth::user()->id;

            $items = $this->transactionItemRepository->save($request);

            DB::commit();

            return $items;

        } catch (Throwable $e) {
            DB::rollback();
            
            return $e;
        }
    }

    public function editItems(Array $request)
    {
        try {
            DB::beginTransaction();

            $request['by_user'] = Auth::user()->id;

            $items = $this->transactionItemRepository->update($request);

            DB::commit();

            return $items;

        } catch (Throwable $e) {
            DB::rollback();

            return $e->getMessage();

        }
    }

    // Files
    public function uploadProofTransaction(Request $request)
    {
        $validated = $request->validate([
            'transaction_id' => 'required|uuid',
            'proof_transaction' => 'required|mimes:jpg,jpeg,png,pdf|max:2048',
        ]);

        try {
            DB::beginTransaction();

            $transaction = $this->transactionRepository->getById($request->transaction_id);
            $transactionDetail = $this->transactionDetailRepository->getById($transaction->details->id);

            if (empty($transaction) || empty($transactionDetail)) {
                return response([
                    'message'  => 'Transaction not found.',
                ], 404);
            }

            if (!is_null($transactionDetail->proof_transaction)) {
                Storage::delete($transactionDetail->proof_transaction);
            }

            $this->transactionDetailRepository->uploadProofTransaction([
                'detail_id' => $transactionDetail->id,
                'proof_transaction' => $request->proof_transaction,
                'by_user' => Auth::user()->id,
            ]);

            DB::commit();

            return response()->json([
                'message' => 'Proof of transaction successfully uploaded.'
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function getProofTransaction(Request $request)
    {
        try {
            $transaction = $this->transactionRepository->getById($request->byId);

            if (empty($transaction)) {
                return response()->json([
                    'message' => 'Transaction not found.'
                ], 404);
            }

            $resource = $this->transactionDetailRepository->getProofTransaction($transaction->details->id);
            $path = storage_path('app/'. $resource->proof_transaction);

            if (!File::exists($path)) {
                return response()->json([
                    'message' => 'Proof payment not found.'
                ], 404);
            }

            $proof_transaction = File::get($path);
            $type = File::mimeType($path);

            return response($proof_transaction, 200)->header('Content-Type', $type);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
            
        }
    }

    public function deleteProofTransaction(Request $request)
    {
        try {
            DB::beginTransaction();

            $transactionDetail = $this->transactionDetailRepository
                ->getById($request->transaction_detail_id);

            $request['by_user'] = Auth::user()->id;
            
            $this->transactionDetailRepository->deleteProofTransaction($request);

            Storage::delete($transactionDetail->proof_transaction);

            DB::commit();

            return response()->json([
                'message' => 'Proof payment successfully deleted.'
            ], 200);
        
        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    // Nodes
    public function getNodesInJson($nodeIds)
    {
        $nodes = array();

        foreach ($nodeIds as $nodeId) {
            $nodeData = $this->nodeRepository->getById($nodeId);

            $node = [
                'id' => $nodeData->id,
                'name' => $nodeData->name,
                'type' => $nodeData->type->name,
            ];

            array_push($nodes, $node);
        }

        return json_encode($nodes);
    }
}
