<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Storage;
use App\Repositories\UserRepository;
use App\Repositories\PersonRepository;
use Throwable;

class UserController extends Controller
{
    protected $userRepository;
    protected $personRepository;

    public function __construct(
        UserRepository $userRepository,
        PersonRepository $personRepository,
    ) {
        $this->userRepository = $userRepository;
        $this->personRepository = $personRepository;
    }

    public function all()
    {
        try {
            $usersData = $this->userRepository->all();
            $users = [];

            foreach ($usersData as $userData) {
                $roles = [];

                foreach ($userData->roles as $role) {
                    array_push($roles, $role->name);
                }

                $user = [
                    'id' => $userData->id,
                    'email' => $userData->email,
                    'is_active' => $userData->is_active,
                    'status' => $userData->status,
                    'is_system_admin' => $userData->is_system_admin,
                    'roles' => $roles,
                    'person' => $userData->person,
                ];

                array_push($users, $user);
            }

            return response()->json($users, 200);
            
        } catch (Throwable $e) {
            return response()->json(['Error' => $e->getMessage()], 500);
            
        }
    }

    public function create(Request $request)
    {
        try {
            DB::beginTransaction();

            $roles = explode(',', $request->roles);

            if ($request->person_id) {
                $validated = $request->validate([
                    'person_id' => 'required|uuid|exists:people,id',
                    'email' => 'required|email|unique:users|max:255',
                    'password' => 'required|min:8',
                    'roles' => 'required|string',
                ]);

                $personId = $request->person_id;

            } else {
                $validated = $request->validate([
                    'name' => 'required|string|max:255',
                    'last_name' => 'required|string|max:255',
                    'email' => 'required|email|unique:users|max:255',
                    'password' => 'required|min:8',
                    'roles' => 'required|string',
                ]);

                $person = $this->personRepository->save([
                    'name' => $request->name,
                    'last_name' => $request->last_name,
                    'by_user' => Auth::user()->id,
                ]);
                $personId = $person->id;
            }

            $user = $this->userRepository->save([
                'person_id' => $personId,
                'email' => $request->email,
                'password' => $request->password,
                'is_system_admin' => $request->is_system_admin,
                'by_user' => Auth::user()->id,
            ]);
            $user->assignRole($roles);

            DB::commit();

            return response()->json([
                'message' => 'Successfully User created.',
                'data' => $user,
            ], 201);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['Error' => $e->getMessage()], 500);
        }
    }

    public function get(Request $request)
    {
        try {
            $userId = $request->byId;

            $userData = $this->userRepository->getById($userId);
            $roles = [];

            if (empty($userData)) {
                return response()->json([
                    'message' => 'User not found.'
                ], 404);
            }

            foreach ($userData->roles as $role) {
                array_push($roles, $role->name);
            }

            $user = [
                'id' => $userData->id,
                'email' => $userData->email,
                'is_active' => $userData->is_active,
                'status' => $userData->status,
                'roles' => $roles,
                'person' => $userData->person,
            ];

            return response()->json($user, 302);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function edit(Request $request)
    {
        $validated = $request->validate([
            'id' => 'required|uuid',
            'email' => 'required|email|max:255',
            'is_system_admin' => 'nullable|boolean',
            'roles' => 'required|string',
        ]);

        try {
            DB::beginTransaction();

            $user = $this->userRepository->getById($request->id);
            $roles = explode(',', $request->roles);

            if (empty($user)) {
                return response()->json([
                    'message' => 'User not found.'
                ], 404);
            }

            if (!Auth::user()->is_system_admin) {
                return response()->json([
                    'message' => 'This user cannot perform this action.'
                ], 403);
            }

            $userUpdated = $this->userRepository->update(
                $user, $request->except('roles'), Auth::user()->id
            );
            $userUpdated->syncRoles($roles);

            DB::commit();

            return response()->json([
                'message' => 'User updated successfully.',
                'data' => $userUpdated,
            ], 200);
            
        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function toggleStatus(Request $request)
    {
        $validated = $request->validate([
            'byId' => 'required|uuid',
        ]);

        try {
            DB::beginTransaction();

            $user = $this->userRepository->getById($request->byId);

            if (empty($user)) {
                return response()->json([
                    'message' => 'User not found.'
                ], 404);
            }

            $userData = $this->userRepository->toggleStatus(
                $request->byId,
                Auth::user()->id,
            );
            $roles = [];

            foreach ($userData->roles as $role) {
                array_push($roles, $role->name);
            }

            $user = [
                'id' => $userData->id,
                'email' => $userData->email,
                'is_active' => $userData->is_active,
                'status' => $userData->status,
                'roles' => $roles,
                'person' => $userData->person,
            ];

            DB::commit();

            return response()->json([
                'message' => 'Successfully modified status.',
                'data' => $user,
            ], 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function changePassword(Request $request)
    {
        $validated = $request->validate([
            'id' => 'required|uuid',
            'password' => 'required|string|min:8|confirmed',
        ]);

        try {
            DB::beginTransaction();

            $user = $this->userRepository->getById($request->id);
            
            if (empty($user)) {
                return response()->json([
                    'message' => 'User not found.'
                ], 404);
            }

            $this->userRepository->changePassword([
                'id' => $request->id,
                'password' => $request->password,
                'by_user' => Auth::user()->id,
            ]);

            DB::commit();

            return response()->json([
                'message' => 'Password updated successfully.',
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function uploadProfilePhoto(Request $request)
    {
        $validated = $request->validate([
            'user_id' => 'required|uuid',
            'profile_picture' => 'required|image|mimes:jpeg,png,jpg|max:2048|dimensions:min_width=200,min_height=200',
        ]);

        try {
            DB::beginTransaction();

            $user = $this->userRepository->getById($request->user_id);

            if (empty($user)) {
                return response()->json([
                    'message' => 'User not found.'
                ], 404);
            }

            if (!is_null($user->profile_photo)) {
                Storage::delete($user->profile_photo);
            }

            $user = $this->userRepository->uploadProfilePhoto([
                'user_id' => $request->user_id,
                'profile_picture' => $request->profile_picture,
                'by_user' => Auth::user()->id,
            ]);

            DB::commit();

            return response()->json([
                'message' => 'profile photo updated successfully.',
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function getProfilePhoto()
    {
        try {
            $resource = $this->userRepository->getProfilePhoto(Auth::user()->id);
            $path = storage_path('app/'. $resource->profile_photo);

            if (!File::exists($path)) {
                return response()->json([
                    'error' => 'Profile photo not found.'
                ], 404);
            }

            $profile_photo = File::get($path);
            $type = File::mimeType($path);

            return response($profile_photo, 200)->header('Content-Type', $type);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
            
        }
    }

    public function revokeSession(Request $request)
    {
        $validated = $request->validate([
            'byId' => 'required|uuid'
        ]);

        try {
            DB::beginTransaction();

            $user = $this->userRepository->getById($request->byId);

            if (empty($user)) {
                return response()->json([
                    'message' => 'User not found.'
                ], 404);
            }

            $user = $this->userRepository->toggleStatus(
                $user->id, Auth::user()->id
            );

            $user->tokens()->delete();

            $user = [
                'id' => $user->id,
                'email' => $user->email,
                'is_active' => $user->is_active,
                'status' => $user->status,
            ];

            DB::commit();

            return response()->json([
                'message' => 'Session successfully revoked.',
                'data' => $user,
            ], 200);
            
        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);

        }
    }
}
