<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use App\Repositories\UserVaultBoxRepository;
use App\Repositories\VaultBoxRepository;
use App\Repositories\VaultBoxHistoryRepository;
use App\Repositories\TransactionRepository;
use App\Repositories\TransactionDetailRepository;
use App\Repositories\TransactionItemRepository;
use App\Repositories\NodeRepository;
use App\Repositories\AccountRepository;
use App\Http\Controllers\TransactionController;
use App\Http\Requests\SaveUserVaultBoxRequest;
use App\Http\Requests\SaveVaultBoxRequest;
use App\Http\Requests\SaveIncomeRequest;
use App\Http\Requests\SaveDispersionRequest;
use App\Http\Requests\SaveRefundRequest;
use Throwable;

class VaultBoxController extends Controller
{
    protected $userVaultBoxRepository;
    protected $vaultBoxRepository;
    protected $vaultBoxHistoryRepository;
    protected $transactionRepository;
    protected $transactionDetailRepository;
    protected $transactionItemRepository;
    protected $nodeRepository;
    protected $accountRepository;

    public function __construct(
        UserVaultBoxRepository $userVaultBoxRepository,
        VaultBoxRepository $vaultBoxRepository,
        VaultBoxHistoryRepository $vaultBoxHistoryRepository,
        TransactionRepository $transactionRepository,
        TransactionDetailRepository $transactionDetailRepository,
        TransactionItemRepository $transactionItemRepository,
        NodeRepository $nodeRepository,
        AccountRepository $accountRepository,
    ) {
        $this->userVaultBoxRepository = $userVaultBoxRepository;
        $this->vaultBoxRepository = $vaultBoxRepository;
        $this->vaultBoxHistoryRepository = $vaultBoxHistoryRepository;
        $this->transactionRepository = $transactionRepository;
        $this->transactionDetailRepository = $transactionDetailRepository;
        $this->transactionItemRepository = $transactionItemRepository;
        $this->nodeRepository = $nodeRepository;
        $this->accountRepository = $accountRepository;
    }

    public function all()
    {
        try {
            if (Auth::user()->is_system_admin) {
                $vaultsBoxesData = $this->vaultBoxRepository->all();
            } else {
                $vaultsBoxesData = $this->vaultBoxRepository
                    ->getAllByUser(Auth::user()->id);
            }

            $vaultsBoxes = $this->getAllNodes($vaultsBoxesData);

            return response()->json($vaultsBoxes, 200);
            
        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
            
        }
    }

    public function get(Request $request)
    {
        try {
            $vaultBox = $this->vaultBoxRepository->getById($request->byId);

            if (empty($vaultBox)) {
                return response()->json([
                    'message' => 'Vault or Box not found.'
                ], 404);
            }

            return response()->json($vaultBox, 302);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function create(SaveVaultBoxRequest $request)
    {
        try {
            DB::beginTransaction();

            $request['by_user'] = Auth::user()->id;
            $vaultBox = $this->vaultBoxRepository->save($request);
            $this->addManager(new SaveUserVaultBoxRequest([
                'user_id' => Auth::user()->id,
                'vault_box_id' => $vaultBox->id
            ]));
            $transactionController = new TransactionController(
                $this->transactionRepository,
                $this->transactionDetailRepository,
                $this->transactionItemRepository,
                $this->vaultBoxRepository,
                $this->vaultBoxHistoryRepository,
                $this->nodeRepository,
                $this->accountRepository,
            );

            if (is_null($vaultBox->belongs_to)) {
                // Add income
                $transaction = $this->addIncome(new SaveIncomeRequest([
                    'id' => $vaultBox->id,
                    'amount' => $request->current_amount,
                    'transaction_method_id' => $request->transaction_method_id,
                    'bank_id' => $request->bank_id,
                    'account_id' => $request->account_id,
                ]));

            } else {
                // Add dispersion
                $transaction = $transactionController->saveDispersion(new SaveDispersionRequest([
                    'total_amount' => $request->current_amount,
                    'vault_box_id' => $vaultBox->belongs_to,
                    'transaction_method_id' => $request->transaction_method_id,
                    'bank_id' => $request->bank_id,
                    'account_id' => $request->account_id,
                    'dispersed_to' => $vaultBox->id,
                ]));
            }

            DB::commit();

            return response()->json([
                'message' => $vaultBox->type->name . ' successfully created.',
                'data' => $vaultBox,
                'transaction' => $transaction->original,
            ], 201);
            
        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function edit(SaveVaultBoxRequest $request)
    {
        try {
            DB::beginTransaction();

            $request['by_user'] = Auth::user()->id;
            $vaultBox = $this->vaultBoxRepository->getById($request->id);

            if (empty($vaultBox)) {
                return response()->json([
                    'message' => 'Vault or Box not found.'
                ], 404);
            }

            $vaultBoxUpdated = $this->vaultBoxRepository->update($vaultBox, $request);

            DB::commit();

            return response()->json([
                'message' => 'Vault or Box successfully updated.',
                'data' => $vaultBoxUpdated,
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return $e->getMessage();
        }
    }

    public function delete(Request $request)
    {
        try {
            DB::beginTransaction();

            $vaultBox = $this->vaultBoxRepository->getById($request->byId);

            if (empty($vaultBox)) {
                return response()->json([
                    'message' => 'Vault or Box successfully removed.',
                ], 404);
            }

            $vaultBoxDeleted = $this->vaultBoxRepository
                ->delete($vaultBox, Auth::user()->id);

            DB::commit();

            return response()->json([
                'message' => 'Vault or Box successfully deleted.',
                'data' => $vaultBoxDeleted,
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function addIncome(SaveIncomeRequest $request)
    {
        try {
            DB::beginTransaction();

            $vault = $this->vaultBoxRepository->getById($request->id);

            if ($vault->level != 0 || !is_null($vault->belongs_to)) {
                return response()->json([
                    'error' => 'This isn\'t a vault or it depends on another.'
                ], 400);
            }

            $request['by_user'] = Auth::user()->id;
            $date = now()->toImmutable()->format('Y-m-d');
            $vaultHistory = $this->vaultBoxHistoryRepository->getOneOpen($vault->id, $date, 'DESC');
            $transaction = $this->transactionRepository->save(array(
                'concept' => 'INGRESO A BÓVEDA',
                'total_amount' => $request->amount,
                'date' => $date,
                'by_user' => Auth::user()->id,
            ));

            if (empty($vaultHistory)) {
                $vaultHistory = $this->vaultBoxHistoryRepository->save(array(
                    'vault_box_id' => $vault->id,
                    'start_date' => $date,
                    'initial_balance' => $vault->current_amount,
                    'total_income' => $request->amount,
                    'total_expenditure' => 0,
                    'total_spent' => 0,
                    'by_user' => $request->by_user,
                ));

            } else {
                $this->vaultBoxHistoryRepository->addIncome(
                    $vaultHistory, $request
                );
            }

            $transactionDetail = $this->transactionDetailRepository->save(array(
                'transaction_id' => $transaction->id,
                'entity_id' => $vault->entity_id,
                'dispersed_to' => $vault->id,
                'dispersed_history_id' => $vaultHistory->id,
                'transaction_method_id' => $request->transaction_method_id,
                'bank_id' => $request->bank_id,
                'account_id' => $request->account_id,
                'type' => 'I',
                'by_user' => $request->by_user,
            ));

            $vault = $this->vaultBoxRepository->addIncome($vault, $request);

            DB::commit();

            return response()->json([
                'message' => 'Income successfully added.',
                'data' => $transaction,
            ], 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function toggleStatus(Request $request)
    {
        try {
            DB::beginTransaction();

            $vaultBox = $this->vaultBoxRepository->getById($request->byId);

            if (empty($vaultBox)) {
                return response()->json([
                    'message' => 'Vault or Box not found.'
                ], 404);
            }

            $vaultBoxStatus = $this->vaultBoxRepository
                ->toggleStatus($vaultBox, Auth::user()->id);

            DB::commit();

            return response()->json([
                'message' => 'Successfully toggled state.',
                'data' => $vaultBoxStatus,
            ], 200);
            
        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function refundBalance(SaveRefundRequest $request)
    {
        try {
            $date = now()->toImmutable()->format('Y-m-d');
            $vaultBox = $this->vaultBoxRepository->getById($request->vault_box_id);
            $vaultBoxHistory = $this->vaultBoxHistoryRepository->getOneOpen($vaultBox->id, $date, 'DESC');
            $dispersedTo = $this->vaultBoxRepository->getById($vaultBox->belongs_to);
            $dispersedHistory = $this->vaultBoxHistoryRepository->getOneOpen($dispersedTo->id, $date, 'DESC');
            $transactionController = new TransactionController(
                $this->transactionRepository,
                $this->transactionDetailRepository,
                $this->transactionItemRepository,
                $this->vaultBoxRepository,
                $this->vaultBoxHistoryRepository,
                $this->nodeRepository,
                $this->accountRepository,
            );

            DB::beginTransaction();
            
            $refund = $transactionController->saveDispersion(new SaveDispersionRequest([
                'is_refund' => true,
                'concept' => 'DEVOLUCIÓN',
                'total_amount' => $request->total_amount,
                'vault_box_id' => $vaultBox->id,
                'dispersed_to' => $dispersedTo->id,
            ]));

            DB::commit();

            return response()->json([
                'message' => '',
                'data' => $refund->original['data']
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    // Historial
    public function getHistoryByVaultBox(String $id)
    {
        try {
            $history = $this->vaultBoxHistoryRepository->getAllOfOne($id);

            return response()->json($history, 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function createHistory(Array $data)
    {
        try {
            DB::beginTransaction();

            $history = $this->userVaultBoxRepository->save($data);

            DB::commit();

            return $history;

        } catch (Throwable $e) {
            return $e->getMessage();
        }
    }

    public function closeHistory(String $id)
    {
        try {
            $date = now()->toImmutable()->format('Y-m-d');
            $vaultBox = $this->vaultBoxRepository->getById($id);

            if (empty($vaultBox)) {
                return response()->json([
                    'message' => 'Vault or Box not found.'
                ], 404);
            }

            $vaultBoxHistory = $this->vaultBoxHistoryRepository->getOneOpen($vaultBox->id, $date, 'ASC');

            if (empty($vaultBoxHistory)) {
                return response()->json([
                    'error' => 'History not found.'
                ], 404);
            }

            DB::beginTransaction();

            $vaultBoxHistory = $this->vaultBoxHistoryRepository->close($vaultBoxHistory, array(
                'end_date' => $date, 
                'final_balance' => $vaultBox->current_amount, 
                'by_user' => Auth::user()->id
            ));

            DB::commit();

            return response()->json([
                'message' => 'History closed successfully.',
                'data' => $vaultBoxHistory,
            ], 200);
            
        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    // Responsables
    public function getManagers(Request $request)
    {
        try {
            $managersData = $this->userVaultBoxRepository->getManagersByVaultBox(
                $request->byVaultBox
            );
            $managers = array();

            if (empty($managersData)) {
                return response()->json([
                    'message' => 'This vault or box there isn\'t any managers.'
                ], 204);
            }

            foreach ($managersData as $managerData) {
                $manager = array(
                    'id' => $managerData->users->id,
                    'email' => $managerData->users->email,
                    'person' => $managerData->users->person,
                    'is_active' => $managerData->users->is_active,
                    'status' => $managerData->users->status,
                );

                array_push($managers, $manager);
            }

            return response()->json($managers, 200);

        } catch (Throwable $e) {
            return response()->json(['error' => $e->getMessage()], 500);

        }
    }

    public function addManager(SaveUserVaultBoxRequest $request)
    {
        try {
            DB::beginTransaction();

            $request['by_user'] = Auth::user()->id;

            $userVaultBox = $this->userVaultBoxRepository->save($request);

            DB::commit();

            return response()->json([
                'message' => 'Manager added to Vault or Box successfully.',
                'data' => $userVaultBox,
            ], 201);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function removeManager(Request $request)
    {
        try {
            DB::beginTransaction();

            $userVaultBox = $this->userVaultBoxRepository->getOne(
                $request->user_id,
                $request->vault_box_id,
            );

            if (empty($userVaultBox)) {
                return response()->json([
                    'message' => 'Manager not found.'
                ], 404);
            }

            $userVaultBoxDeleted = $this->userVaultBoxRepository
                ->delete($userVaultBox, Auth::user()->id);

            DB::commit();

            return response()->json([
                'message' => 'Manager successfully removed.',
                'data' => $userVaultBoxDeleted,
            ], 200);

        } catch (Throwable $e) {
            DB::rollback();

            return response()->json(['error' => $e->getMessage()], 500);
        }
    }
}
